"""
Module that summarizes an assignment group by maximizing grade and date. This 
module is the same as `canvaslms.grades.disjunctmax`, but also includes 
ungraded surveys (for instance quiz with points, where the number of points is 
ignored). Consequently all assignments must have a date.

This function also doen't fail when there is a grade other than A--F or P/F 
present. Such grades are all treated as F.
"""

import datetime as dt
from canvaslms.grades.disjunctmax import grade_max

def summarize(user, assignments_list):
  """Extracts user's submissions for assignments in assingments_list to 
  summarize results into one grade and a grade date. Summarize by disjunctive 
  maximum."""

  grades = []
  dates = []

  for assignment in assignments_list:
    submission = assignment.get_submission(user)
    grade = submission.grade

    if grade is None or grade not in "ABCDEPF":
      grade = "F"

    grades.append(grade)

    grade_date = submission.submitted_at or submission.graded_at

    if grade_date:
      grade_date = dt.date.fromisoformat(grade_date.split("T")[0])
      dates.append(grade_date)

  if len(dates) < len(grades):
    return ("F", None)
  return (grade_max(grades), max(dates))

def summarize_group(assignments_list, users_list):
  """Summarizes a particular set of assignments (assignments_list) for all
  users in users_list"""

  for user in users_list:
    grade, grade_date = summarize(user, assignments_list)
    yield (user, grade, grade_date)



import arrow
import canvaslms.hacks.canvasapi
import csv
import datetime
import re
import sys

def courses_command(config, canvas, args):
  """Prints the users list of courses in CSV format to stdout"""
  output = csv.writer(sys.stdout, delimiter=args.delimiter)

  course_list = filter_courses(canvas, args.regex)

  if not args.all:
    is_current_course = lambda x: \
        x.start_at is None \
        or (x.end_at is None and \
          arrow.get(x.start_at)-arrow.now().shift(years=-1) \
            > datetime.timedelta(0)) \
        or x.end_at is not None and arrow.get(x.end_at) > arrow.now()
    course_list = filter(is_current_course, course_list)

  for course in course_list:
    output.writerow([
      course.course_code,
      course.name,
      course.start_at,
      course.end_at
    ])
def filter_courses(canvas, regex):
  courses = canvas.get_courses()
  p = re.compile(regex)
  for course in courses:
    if p.search(course.name):
      yield course
    elif p.search(course.course_code):
      yield course
    elif p.search(str(course.id)):
      yield course
def add_course_option(parser, required=False):
  """Adds the -c option to argparse parser to filter out courses"""
  parser.add_argument("-c", "--course", required=required, default=".*",
    help="Regex matching courses on title, course code or Canvas ID, "
      "default: '.*'")

def process_course_option(canvas, args):
  """Processes -c option, returns a list of courses"""
  course_list = filter_courses(canvas, args.course)
  return list(course_list)

def add_command(subp):
  """Adds the subcomand and its options to argparse subparser subp"""
  courses_parser = subp.add_parser("courses",
    help="Lists your courses",
    description="Lists your courses. Output, CSV-format: "
      "<course-code> <course-name> <start-time> <end-time>")
  courses_parser.set_defaults(func=courses_command)
  courses_parser.add_argument("regex",
    default=".*", nargs="?",
    help="Regex for filtering courses, default: '.*'")
  courses_parser.add_argument("-a", "--all",
    action="store_true", default=False,
    help="List all courses, by default list only current courses.")



import canvasapi.submission

import canvaslms.cli.assignments as assignments
import canvaslms.cli.users as users
import canvaslms.hacks.canvasapi

import argparse
import csv
import json
import os
import pypandoc
import re
import rich.console
import rich.markdown
import sys
import urllib.request

def submissions_command(config, canvas, args):
  assignment_list = assignments.process_assignment_option(canvas, args)
  if args.ungraded:
    submissions = list_ungraded_submissions(assignment_list)
  else:
    submissions = list_submissions(assignment_list)

  if args.user or args.category or args.group:
    user_list = users.process_user_or_group_option(canvas, args)
    submissions = filter_submissions(submissions, user_list)

  output = csv.writer(sys.stdout, delimiter=args.delimiter)

  for submission in submissions:
    if args.login_id:
      output.writerow(format_submission_short_unique(submission))
    else:
      output.writerow(format_submission_short(submission))
def speedgrader(submission):
  """Returns the SpeedGrader URL of the submission"""
  speedgrader_url = submission.preview_url

  speedgrader_url = re.sub("assignments/",
    "gradebook/speed_grader?assignment_id=",
    speedgrader_url)

  speedgrader_url = re.sub("/submissions/",
    "&student_id=",
    speedgrader_url)

  speedgrader_url = re.sub(r"\?preview.*$", "", speedgrader_url)

  return speedgrader_url
def submission_command(config, canvas, args):
  submission_list = process_submission_options(canvas, args)
  console = rich.console.Console()
  for submission in submission_list:
    output = format_submission(submission)

    if sys.stdout.isatty():
      pager = ""
      if "MANPAGER" in os.environ:
        pager = os.environ["MANPAGER"]
      elif "PAGER" in os.environ:
        pager = os.environ["PAGER"]

      styles = False
      if "less" in pager and ("-R" in pager or "-r" in pager):
        styles = True
      with console.pager(styles=styles):
        console.print(rich.markdown.Markdown(output,
                                             code_theme="manni"))
    else:
      print(output)
def add_submission_options(parser, required=False):
  try:
    assignments.add_assignment_option(parser, required=required)
  except argparse.ArgumentError:
    pass

  try:
    users.add_user_or_group_option(parser, required=required)
  except argparse.ArgumentError:
    pass

  submissions_parser = parser.add_argument_group("filter submissions")
  try: # to protect from this option already existing in add_assignment_option
    submissions_parser.add_argument("-U", "--ungraded", action="store_true",
      help="Only ungraded submissions.")
  except argparse.ArgumentError:
    pass

def process_submission_options(canvas, args):
  assignment_list = assignments.process_assignment_option(canvas, args)
  user_list = users.process_user_or_group_option(canvas, args)

  if args.ungraded:
    submissions = list_ungraded_submissions(assignment_list,
      include=["submission_history", "submission_comments", 
      "rubric_assessment"])
  else:
    submissions = list_submissions(assignment_list,
      include=["submission_history", "submission_comments", 
      "rubric_assessment"])

  return list(filter_submissions(submissions, user_list))
def list_submissions(assignments, include=["submission_comments"]):
  for assignment in assignments:
    submissions = assignment.get_submissions(include=include)
    for submission in submissions:
      submission.assignment = assignment
      yield submission

def list_ungraded_submissions(assignments, include=["submisson_comments"]):
  for assignment in assignments:
    submissions = assignment.get_submissions(bucket="ungraded",
      include=include)
    for submission in submissions:
      if submission.submitted_at and (submission.graded_at is None or
          not submission.grade_matches_current_submission):
        submission.assignment = assignment
        yield submission
def filter_submissions(submission_list, user_list):
  user_list = set(user_list)

  for submission in submission_list:
    for user in user_list:
      if submission.user_id == user.id:
        submission.user = user
        yield submission
        break
def format_submission_short(submission):
  return [submission.assignment.course.course_code, submission.assignment.name,
    submission.user.name, submission.grade,
    submission.submitted_at, submission.graded_at]

def format_submission_short_unique(submission):
  return [submission.assignment.course.course_code, submission.assignment.name,
    submission.user.login_id, submission.grade,
    submission.submitted_at, submission.graded_at]
def format_submission(submission):
  """Formats submission for printing to stdout"""
  student = submission.assignment.course.get_user(submission.user_id)

  formatted_submission = ""

  formatted_submission += format_section(
    "Metadata",
    f"{submission.assignment.course.course_code} > {submission.assignment.name}"
    f"\n\n"
    f" - Student: {student.name} "
      f"({student.login_id or None}, {submission.user_id})\n"
    f" - Submission ID: {submission.id}\n"
    f" - Submitted (graded): {submission.submitted_at} "
      f"({submission.graded_at})\n"
    f" - Grade: {submission.grade} ({submission.score})\n"
    f" - SpeedGrader: {speedgrader(submission)}")
  try:
    if submission.rubric_assessment:
      formatted_submission += format_section(
        "Rubric assessment",
        format_rubric(submission))
  except AttributeError:
    pass
  try:
    if submission.submission_comments:
      body = ""
      for comment in submission.submission_comments:
        body += f"{comment['author_name']} ({comment['created_at']}):\n\n"
        body += comment["comment"] + "\n\n"
      formatted_submission += format_section("Comments", body)
  except AttributeError:
    pass
  try:
    if submission.body:
      formatted_submission += format_section("Body", submission.body)
  except AttributeError:
    pass
  try:
    if submission.submission_data:
      formatted_submission += format_section(
        "Quiz answers",
        json.dumps(submission.submission_data, indent=2))
  except AttributeError:
    pass
  try:
    for attachment in submission.attachments:
      content_type = ct_to_md(attachment["content-type"])
      if not content_type:
        continue

      contents = urllib.request.urlopen(attachment["url"]).read().decode("utf8")

      formatted_submission += format_section(
        attachment["filename"],
        f"```{content_type}\n{contents}\n```"
      )
  except AttributeError:
    pass
  try:
    if submission.submission_history:
      for prev_submission in submission.submission_history:
        prev_submission = canvasapi.submission.Submission(
          submission._requester, prev_submission)
        prev_submission.assignment = submission.assignment

        formatted_submission += "\n\n" + format_submission(prev_submission)
  except AttributeError:
    pass

  return formatted_submission
def format_section(title, body):
  return f"\n# {title}\n\n{body}\n\n"
def ct_to_md(content_type):
  """
  Takes a content type, returns Markdown code block type.
  Returns None if not possible.
  """
  if content_type.startswith("text/"):
    content_type = content_type[len("text/"):]
  else:
    return None

  if content_type.startswith("x-"):
    content_type = content_type[2:]
  if content_type == "python-script":
    content_type = "python"

  return content_type
def format_rubric(submission):
  """Format the rubric assessment of the `submission` in readable form."""

  result = ""

  for crit_id, rating_data in submission.rubric_assessment.items():
    criterion = get_criterion(crit_id, submission.assignment.rubric)
    rating = get_rating(rating_data["rating_id"], criterion)

    result += f"{criterion['description']}: {rating['description']} " \
      f"({rating['points']})\n"

    if rating_data["comments"]:
      result += f"Comments: {rating_data['comments']}\n"

    result += "\n"

  return result.strip()
def get_criterion(criterion_id, rubric):
  """Returns criterion with ID `criterion_id` from rubric `rubric`"""
  for criterion in rubric:
    if criterion["id"] == criterion_id:
      return criterion

  return None
def get_rating(rating_id, criterion):
  """Returns rating with ID `rating_id` from rubric criterion `criterion`"""
  for rating in criterion["ratings"]:
    if rating["id"] == rating_id:
      return rating

  return None

def add_command(subp):
  """Adds the submissions and submission commands to argparse parser subp"""
  add_submissions_command(subp)
  add_submission_command(subp)

def add_submissions_command(subp):
  """Adds submissions command to argparse parser subp"""
  submissions_parser = subp.add_parser("submissions",
      help="Lists submissions of an assignment",
      description="Lists submissions of assignment(s). Output format: "
        "<course code> <assignment name> <user> <grade> "
        "<submission date> <grade date>")
  submissions_parser.set_defaults(func=submissions_command)
  assignments.add_assignment_option(submissions_parser)
  add_submission_options(submissions_parser)
  submissions_parser.add_argument("-l", "--login-id",
    help="Print login ID instead of name.",
    default=False, action="store_true")

def add_submission_command(subp):
  """Adds submission command to argparse parser subp"""
  submission_parser = subp.add_parser("submission",
    help="Prints information about a submission",
    description="Prints data about matching submissions, "
      "including submission and grading time, any text-based attachments.")
  submission_parser.set_defaults(func=submission_command)
  add_submission_options(submission_parser)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['canvaslms', 'canvaslms.cli', 'canvaslms.grades', 'canvaslms.hacks']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'argcomplete>=2.0.0,<3.0.0',
 'arrow>=1.2.3,<2.0.0',
 'cachetools>=5.2.0,<6.0.0',
 'canvasapi<3.0.0',
 'keyring>=23.11.0,<24.0.0',
 'pypandoc>=1.10,<2.0',
 'rich>=13.0.0,<14.0.0']

entry_points = \
{'console_scripts': ['canvaslms = canvaslms.cli:main']}

setup_kwargs = {
    'name': 'canvaslms',
    'version': '2.7',
    'description': 'Command-line interface to Canvas LMS',
    'long_description': '# canvaslms: A CLI to Canvas LMS.\n\nThis program provides a command-line interface for Canvas. The command\nis `canvaslms` and it has several subcommands in the same style as Git.\n`canvaslms` provides output in a format useful for POSIX tools, this\nmakes automating tasks much easier.\n\nLet\'s consider how to grade students logging into the student-shell SSH\nserver. We store the list of students\' Canvas and KTH IDs in a file.\n\n``` {.text}\ncanvaslms users -c DD1301 -s | cut -f 1,2 > students.csv\n```\n\nThen we check who has logged into student-shell.\n\n``` {.text startFrom="2"}\nssh student-shell.sys.kth.se last | cut -f 1 -d " " | sort | uniq \\\n  > logged-in.csv\n```\n\nFinally, we check who of our students logged in.\n\n``` {.text startFrom="4"}\nfor s in $(cut -f 2 students.csv); do\n  grep $s logged-in.csv && \\\n```\n\nFinally, we can set their grade to P and add the comment "Well done!" in\nCanvas. We set the grades for the two assignments whose titles match the\nregular expression `(Preparing the terminal|The terminal)`.\n\n``` {.text startFrom="6"}\n    canvaslms grade -c DD1301 -a "(Preparing the terminal|The terminal)" \\\n      -u $(grep $s students.csv | cut -f 1) \\\n      -g P -m "Well done!"\ndone\n```\n\n## Installation\n\nJust install the PyPI package:\n```\npython3 -m pip install canvaslms\n```\nSome subcommands use `pandoc`, so you will likely have to [install \npandoc][pandoc] on your system manually.\n\n[pandoc]: https://pandoc.org/installing.html\n',
    'author': 'Daniel Bosk',
    'author_email': 'dbosk@kth.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/dbosk/canvaslms',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import argparse
import canvasapi
import canvaslms.cli.courses as courses
import canvaslms.hacks.canvasapi
import csv
import os
import pypandoc
import re
import rich.console
import rich.markdown
import sys

def add_assignment_option(parser, ungraded=True, required=False):
  try:
    courses.add_course_option(parser, required=required)
  except argparse.ArgumentError:
    pass

  if ungraded:
    parser.add_argument("-U", "--ungraded", action="store_true",
      help="Filter only assignments with ungraded submissions.")

  parser = parser.add_mutually_exclusive_group(required=required)

  parser.add_argument("-a", "--assignment",
    required=False, default=".*",
    help="Regex matching assignment title or Canvas identifier, "
      "default: '.*'")

  parser.add_argument("-A", "--assignment-group",
    required=False, default="",
    help="Regex matching assignment group title or Canvas identifier.")

def process_assignment_option(canvas, args):
  course_list = courses.process_course_option(canvas, args)
  assignments_list = []

  for course in course_list:
    try:
      ungraded = args.ungraded
    except AttributeError:
      ungraded = False

    all_assignments = list(filter_assignments([course],
      args.assignment,
      ungraded=ungraded))

    try:
      assignm_grp_regex = args.assignment_group
    except AttributeError:
      print("default to .* for group")
      assignm_grp_regex = ".*"

    assignment_groups = filter_assignment_groups(course, assignm_grp_regex)

    for assignment_group in assignment_groups:
      assignments_list += list(filter_assignments_by_group(
          assignment_group, all_assignments))
  return list(assignments_list)
def assignments_command(config, canvas, args):
  output = csv.writer(sys.stdout, delimiter=args.delimiter)
  assignment_list = process_assignment_option(canvas, args)
  for assignment in assignment_list:
    output.writerow([assignment.assignment_group.name, assignment.name])
def filter_assignment_groups(course, regex):
  """Returns all assignment groups of course whose name matches regex"""
  name = re.compile(regex)
  return filter(
    lambda group: name.search(group.name),
    course.get_assignment_groups())
def filter_assignments_by_group(assignment_group, assignments):
  """Returns elements in assignments that are part of assignment_group"""
  for assignment in assignments:
    if assignment.assignment_group_id == assignment_group.id:
      assignment.assignment_group = assignment_group
      yield assignment
def assignment_command(config, canvas, args):
  console = rich.console.Console()

  assignment_list = process_assignment_option(canvas, args)
  for assignment in assignment_list:
    output = format_assignment(assignment)

    if sys.stdout.isatty():
      pager = ""
      if "MANPAGER" in os.environ:
        pager = os.environ["MANPAGER"]
      elif "PAGER" in os.environ:
        pager = os.environ["PAGER"]

      styles = False
      if "less" in pager and ("-R" in pager or "-r" in pager):
        styles = True
      with console.pager(styles=styles, links=True):
        console.print(rich.markdown.Markdown(output,
                                             code_theme="manni"))
    else:
      print(output)
def format_assignment(assignment):
  """Returns an assignment formatted for the terminal"""
  instruction = pypandoc.convert_text(
    assignment.description, "md", format="html")

  return f"""# {assignment.name}

{instruction}

URL: {assignment.html_url}
Submissions: {assignment.submissions_download_url}"""
def list_assignments(assignments_containers, ungraded=False):
  """Lists all assignments in all assignments containers (courses or 
  assignement groups)"""
  for container in assignments_containers:
    if isinstance(container, canvasapi.course.Course):
      course = container
    elif isinstance(container, canvasapi.assignment.AssignmentGroup):
      assignment_group = container
      course = assignment_group.course

    if ungraded:
      assignments = container.get_assignments(bucket="ungraded")
    else:
      assignments = container.get_assignments()

    for assignment in assignments:
      try:
        assignment.course = course
      except NameError:
        pass

      try:
        assignment.assignment_group = assignment_group
      except NameError:
        pass

      yield assignment

def list_ungraded_assignments(assignments_containers):
  return list_assignments(assignments_containers, ungraded=True)
def filter_assignments(assignments_containers, regex, ungraded=False):
  """Returns all assignments from assignments_container whose
  title matches regex"""
  p = re.compile(regex)
  for assignment in list_assignments(assignments_containers, ungraded=ungraded):
    if p.search(assignment.name):
      yield assignment
    elif p.search(str(assignment.id)):
      yield assignment

def add_command(subp):
  """Adds the subcommands assignments and assignment to argparse parser subp"""
  add_assignments_command(subp)
  add_assignment_command(subp)

def add_assignments_command(subp):
  """Adds the assignments subcommand to argparse parser subp"""
  assignments_parser = subp.add_parser("assignments",
      help="Lists assignments of a course",
      description="Lists assignments of a course. "
        "Output, CSV-format: <assignment group> <assignment name>")
  assignments_parser.set_defaults(func=assignments_command)
  add_assignment_option(assignments_parser)

def add_assignment_command(subp):
  """Adds the assignment subcommand to argparse parser subp"""
  assignment_parser = subp.add_parser("assignment",
      help="Lists assignment details",
      description="Lists assignment details")
  assignment_parser.set_defaults(func=assignment_command)
  add_assignment_option(assignment_parser)
